#include "ConfigManager.h"



ConfigManager::~ConfigManager()
{
	Clear();
}


bool ConfigManager::Initialize()
{
	FileReader configReader;

	try{
		configReader.Open("config.ini", FileReader::ASCII);
	}

	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}

	string fieldName, fieldValue;

	try
	{
		while (configReader.EoF () == false)
		{
			if (configReader.GetLine (fieldName, 64, '=') == true)
			{
				Trim (fieldName);

				if (configReader.EoF () == false &&
						configReader.GetLine (fieldValue, 512, '\n') == true)
				{
					
					Trim(fieldValue);
					if (fieldName.compare ("initialize") == 0)
						initialize = fieldValue;
					else if (fieldName.compare ("wordListFilePath") == 0)
						wordListFilePath = fieldValue;
					else if (fieldName.compare ("hashTableFilePath") == 0)
						hashTableFilePath = fieldValue;
					else if (fieldName.compare ("bigramMatrixFilePath") == 0)
						bigramMatrixFilePath = fieldValue;
					else if (fieldName.compare ("bigramPLFilePath") == 0)
						bigramPLFilePath = fieldValue;
					else if (fieldName.compare ("unigramListsFilePath") == 0)
						unigramListsFilePath = fieldValue;
					else if (fieldName.compare ("unigramPLFilePath") == 0)
						unigramPLFilePath = fieldValue;
					else if (fieldName.compare("urduAlphabetFilePath") == 0)
						urduAlphabetFilePath = fieldValue;	
					else if (fieldName.compare("soundexFilePath") == 0)
						soundexFilePath = fieldValue;	
					else if (fieldName.compare("shapexFilePath") == 0)
						shapexFilePath = fieldValue;	
				}
			}
		}

		configReader.Close();
	}
	catch (IOException& ex)
	{
		// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file reading problem occurred.");
		throw; // re-throw the same exception object
	}
	
	return true;	
}

bool ConfigManager::writeConfig()
{

	FileWriter configWriter;
	char endl[2] = {13,10};
	
	try{
		configWriter.Open("config.ini", FileWriter::ASCII);
	}
	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}

	try{
		configWriter.Write("initialize = NO");
		configWriter.Write(endl,2);
		
		configWriter.Write("wordListFilePath = ");
		configWriter.Write(wordListFilePath);
		configWriter.Write(endl,2);

		configWriter.Write("hashTableFilePath = ");
		configWriter.Write(hashTableFilePath);
		configWriter.Write(endl,2);

		configWriter.Write("bigramMatrixFilePath = ");
		configWriter.Write(bigramMatrixFilePath);
		configWriter.Write(endl,2);

		configWriter.Write("bigramPLFilePath = ");
		configWriter.Write(bigramPLFilePath);
		configWriter.Write(endl,2);

		configWriter.Write("unigramListsFilePath = ");
		configWriter.Write(unigramListsFilePath);
		configWriter.Write(endl,2);

		configWriter.Write("unigramPLFilePath = ");
		configWriter.Write(unigramPLFilePath);
		configWriter.Write(endl,2);

		configWriter.Write("urduAlphabetFilePath = ");
		configWriter.Write(urduAlphabetFilePath);
		configWriter.Write(endl,2);

		configWriter.Write("soundexFilePath = ");
		configWriter.Write(soundexFilePath);
		configWriter.Write(endl,2);

		configWriter.Write("shapexFilePath = ");
		configWriter.Write(shapexFilePath);
		configWriter.Write(endl,2);


	}
	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}

	return true;
}

void ConfigManager::Clear()
{
	initialize.erase();
	wordListFilePath.erase();
	hashTableFilePath.erase();
	bigramMatrixFilePath.erase();
	bigramPLFilePath.erase();
	unigramListsFilePath.erase();
	unigramPLFilePath.erase();

}