#include <iostream.h>

#include "SpellChecker.h"
#pragma warning(disable: 4786)


bool SpellChecker::Initialize()
{
	
	if (config.Initialize()==false) return false;
	if (charmap.Initialize(config.urduAlphabetFilePath)==false) return false;

	plResource.SetNumberOfChars(charmap.GetTotalNumOfChars());



	if(config.initialize.compare("YES")==0)
	{
		hashtable.Build(config.wordListFilePath,charmap);
		plResource.Initialize(hashtable, charmap);
		hashtable.Save(config.hashTableFilePath);
		plResource.Save(config);
		config.writeConfig();
	}	
	else if (config.initialize.compare("NO")==0)
	{
		hashtable.Load(config.hashTableFilePath);
		plResource.Load(config);
	}


//	hashtable.Load(config.hashTableFilePath);



	InitializeSoundex(config.soundexFilePath);
	InitializeShapex(config.shapexFilePath);
		
	try{
		bigramPLReader.Open(config.bigramPLFilePath.c_str(),FileReader::UNI);
		unigramPLReader.Open(config.unigramPLFilePath.c_str(),FileReader::UNI);
	}
	catch (IOException& ex)
	{
		// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file reading problem occurred.");
		throw; // re-throw the same exception object
	}


	return true;
}


bool SpellChecker::SpellCheck(wstring uword, vector<wstring> &suggestedList)
{

	suggestedList.clear();

	uword = charmap.Validate(uword);

	if(uword.size()<1)
		return false;
	
	if(hashtable.Search(uword) == true)
		return false;
	else{

		wstring ErrWord,TransposeWord;

		vector<HashNode> res;
		int ErrLength, CorLength, Distance, TransposeWordId,row,col;
		vector<int> Candidates;
		vector<int> * RankBucket;
		RankBucket=new vector<int>[40];
		vector<HashNode> RankResult[3];

		PostList PostingList;
		Byte *WordGramCountList;
		RecordPtr *BigramAddresses;
		//ErrWord=RemoveDiacritics(uword);
		ErrWord = uword;
		ErrLength=ErrWord.length();
		

		BigramAddresses= new RecordPtr[ErrLength];
		WordGramCountList= new Byte[hashtable.getArrSize()]; 
		memset((void *)WordGramCountList,0,hashtable.getArrSize());
		vector <HashNode> SearchResults;
		HashNode hn;
		int alphaSize;
		alphaSize = charmap.GetTotalNumOfChars();

	
		if (ErrLength == 1)
			return false;


		if (ErrLength>=3)
		{
			int i = 0;
			
			int k;
			vector<int>::iterator iter;
			
			//Checking for Transposition error
			for(i=0;i<=ErrLength-2;i++)
			{
				TransposeWord = ErrWord;
				TransposeWord[i] = ErrWord[i+1];
				TransposeWord[i+1] = ErrWord[i];
				hashtable.Search(TransposeWord,SearchResults);
						
				if (SearchResults.size()>0)
					for(int j=0;j<SearchResults.size();j++)
					{
						if (WordGramCountList[SearchResults[j].Index]==0)
						{
							RankBucket[1].push_back(SearchResults[j].Index);
							WordGramCountList[SearchResults[j].Index]=1;
						}
							
					}
			}


			for(i=0;i<=ErrLength-2;i++)
			{
				row = charmap.GetIndex(ErrWord[i]);
				col = charmap.GetIndex(ErrWord[i+1]);
				if(row <0 || row>= alphaSize || col <0 || col >=alphaSize){
					//throw(OutOfBoundException (__FILE__, __LINE__, INDEX_OUT_OF_BOUND, "array index out of bound excception"));
				}
				else
				{
					BigramAddresses[i]=plResource.bigramMatrix[row][col];	
				}
			}
		
			sort(BigramAddresses,BigramAddresses+ErrLength-1  );

			if(BigramAddresses[ErrLength-2].count)		
			{
				PostingList.data=new int[BigramAddresses[ErrLength-2].count];
				bigramPLReader.Read((char *)PostingList.data,BigramAddresses[ErrLength-2].offset,BigramAddresses[ErrLength-2].count*sizeof(int));
				

				
				for(i=0;i<BigramAddresses[ErrLength-2].count;i++)
				{
					if (WordGramCountList[PostingList.data[i]]==0)
					{
						WordGramCountList[PostingList.data[i]]=1;
						Candidates.push_back(PostingList.data[i]);
					}
				}

				delete PostingList.data;
				PostingList.data = NULL;
				PostingList.size = 0;
			}
			
			for(i=ErrLength-3;i>=0;i--)
			{
				if(BigramAddresses[i].count)
				{
					PostingList.data=new int[BigramAddresses[i].count];
					bigramPLReader.Read((char *)PostingList.data,BigramAddresses[i].offset,BigramAddresses[i].count*sizeof(int));
					
					for(int j=0;j<BigramAddresses[i].count;j++)
					{
						if (WordGramCountList[PostingList.data[j]]==0)
						{
							Candidates.push_back (PostingList.data[j]);
							WordGramCountList[PostingList.data[j]]=1;
						}				
					}
					
					delete PostingList.data;
					PostingList.data = NULL;
					PostingList.size = 0;
				}
				
			}

		}


		if (ErrLength <= 3)
		{
			int i = 0;

			//get unigrams// posting list addresses for words of length less than 3
			for(i = 0;i<ErrLength;i++)
			{
				row = charmap.GetIndex(ErrWord[i]);
				if(row <0 || row>= alphaSize ){
					//throw(OutOfBoundException (__FILE__, __LINE__, INDEX_OUT_OF_BOUND, "array index out of bound excception"));
				}
				else
				{
					BigramAddresses[i] = plResource.unigram[row];	
				}
			}

			sort(BigramAddresses,BigramAddresses+ErrLength  );

			if(BigramAddresses[ErrLength-1].count)
			{
				PostingList.data=new int[BigramAddresses[ErrLength-1].count];

				unigramPLReader.Read((char *)PostingList.data,BigramAddresses[ErrLength-1].offset,BigramAddresses[ErrLength-1].count*sizeof(int));
					

				for(i=0;i<BigramAddresses[ErrLength-1].count;i++)
				{
					if(WordGramCountList[PostingList.data[i]]==0)
					{
						WordGramCountList[PostingList.data[i]]=1;
						Candidates.push_back(PostingList.data[i]);
					}
				
				}

				delete PostingList.data;
				PostingList.data = NULL;
				PostingList.size = 0;
			}
			
			for(i=ErrLength-2;i>=0;i--)
			{
				if(BigramAddresses[i].count)
				{
				PostingList.data=new int[BigramAddresses[i].count];
				unigramPLReader.Read((char *)PostingList.data,BigramAddresses[i].offset,BigramAddresses[i].count*sizeof(int));
				for(int j=0;j<BigramAddresses[i].count;j++)
				{
					if (WordGramCountList[PostingList.data[j]]==0)
					{
						Candidates.push_back (PostingList.data[j]);
						WordGramCountList[PostingList.data[j]]=1;
					}
				}
					
				delete PostingList.data;
				PostingList.data = NULL;
				PostingList.size = 0;
				}
					
			}
				
		}
		
		int i = 0;

		for (i = 0; i<Candidates.size();i++)
		{
			hashtable.GetNodeAtIndex (Candidates[i],hn);
						
		
			if(hn.HashKey.length()<= ErrWord.length()+3 && hn.HashKey.length()>= ErrWord.length()-3)
				Distance=EditDistance(ErrWord,hn.HashKey);
			else 
				Distance = 39;


		
			if(Distance>39)
				Distance=39;
			RankBucket[Distance].push_back(Candidates[i]);	  
		}


		for(i=0;i<40;i++)
		{
			if(RankBucket[i].size()>0)
				break;
		}

		int FarthestNeighbour;

		if(i<2)
			FarthestNeighbour=i;
		else
			if(ErrLength>5)
				FarthestNeighbour=2;
			else
				FarthestNeighbour=1;
		
		vector< HashNode > left;
		int ErrSoundex=GetSoundex(ErrWord);
		int ErrShapex = GetShapex(ErrWord);
		int tsoundex,tshapex;


		for(i=1;i<=FarthestNeighbour;i++)
		{
			left.clear();
			int j =0;
			for(j = 0;j< RankBucket[i].size();j++)
			{
				
				hashtable.GetNodeAtIndex (RankBucket[i][j],hn);

				hn.Index = RankBucket[i][j];//why is the index set here

				//Priority is given to those corrections whose soundex matches
				
				// change by Shakil: hashtable stores words with spaces; remove spaces before computing soundex 
				//if(GetSoundex(RemoveWhitespaces(hn.HashKey))==ErrSoundex)

				tsoundex = GetSoundex(hn.HashKey);
				tshapex = GetShapex(hn.HashKey);

				if(tsoundex==ErrSoundex && tshapex==ErrShapex)
					RankResult[0].push_back(hn);
				else if(tsoundex==ErrSoundex)
					RankResult[1].push_back(hn);
				else if(tshapex==ErrShapex)
					RankResult[1].push_back(hn);
				else
					RankResult[2].push_back(hn);

/*
				if(GetSoundex(hn.HashKey)==ErrSoundex)
					res.push_back(hn);
				else
					left.push_back(hn);*/
			}
			/*for(j = 0;j< left.size();j++)
				res.push_back(left[j]);*/

			
			for(int m=0;m<3;m++)
			{
				sort(RankResult[m].begin(),RankResult[m].end());

				//cout<<"\nAT "<<m<<flush;

				for(int n=RankResult[m].size()-1;n>=0;n--)
				{
					suggestedList.push_back(RankResult[m][n].HashKey);
				//	cout<<"\n"<<RankResult[m][n].Frequency<<flush;
				}
			//	RankResult[i].clear();
			}
						

		}

		for(int j = 0;j< RankBucket[FarthestNeighbour+1].size();j++)
		{
			hashtable.GetNodeAtIndex (RankBucket[FarthestNeighbour+1][j],hn);
			hn.Index = RankBucket[FarthestNeighbour+1][j];//why is the index set here

			tsoundex = GetSoundex(hn.HashKey);
			tshapex = GetShapex(hn.HashKey);


			if(tsoundex==ErrSoundex /*&& tshapex==ErrShapex*/)
				res.push_back(hn);

		}


		sort(res.begin(),res.end());

		for(i=res.size()-1;i>=0;i--)
		{
			suggestedList.push_back(res.at(i).HashKey);
	//		cout<<"\n"<<res[i].Frequency<<flush;
		}
		
/*		sort(RankResult[2].begin(),RankResult[2].end());

		for(i=RankResult[2].size()-1; i>=0 ; i--)
			suggestedList.push_back(RankResult[2][i].HashKey);*/

		



		delete []WordGramCountList;
		delete []BigramAddresses;
		delete []RankBucket;
		return true;

	}

	
	return false;
}


int SpellChecker::GetSoundex(wstring word)
{
	int soundex_code=0;

	for(int i=0;i<word.length();i++)
	{
		if(Soundex[word[i]-1536]!=0)
			soundex_code=soundex_code*10+Soundex[word[i]-1536];
	}

	while(soundex_code<1000&&soundex_code>0)
		soundex_code=soundex_code*10;

	return soundex_code;
}

int SpellChecker::GetShapex(wstring word)
{
	int shapex_code=0;

	for(int i=0;i<word.length();i++)
	{
		if(Shapex[word[i]-1536]!=0)
			shapex_code=shapex_code*10+Shapex[word[i]-1536];
	}

	while(shapex_code<1000&&shapex_code>0)
		shapex_code=shapex_code*10;

	return shapex_code;
}


bool SpellChecker::InitializeSoundex(string filename)
{

	for(int i = 0;i<300;i++)
		Soundex[i] = 0;
	

	FileReader soundexReader;

	try{
		soundexReader.Open(filename.c_str(), FileReader.ASCII);
	}

	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}

	string unicode, soundexValue;
	char *endptr;

	try{
		
		while (soundexReader.EoF()==false)
		{
			if(soundexReader.GetLine(unicode,10,'=')==true)
				soundexReader.GetLine(soundexValue,5,'\n');

			Trim(unicode);
			Trim(soundexValue);

			Soundex[strtol(unicode.data(),&endptr,16) - 0x600] = atoi(soundexValue.data());

		}
	}
	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}



	/*********************************/
	return true;
}

bool SpellChecker::InitializeShapex(string filename)
{

	for(int i = 0;i<300;i++)
		Shapex[i] = 0;
	

	FileReader shapexReader;

	try{
		shapexReader.Open(filename.c_str(), FileReader.ASCII);
	}

	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}

	string unicode, shapexValue;
	char *endptr;

	try{
		
		while (shapexReader.EoF()==false)
		{
			if(shapexReader.GetLine(unicode,10,'=')==true)
				shapexReader.GetLine(shapexValue,5,'\n');

			Trim(unicode);
			Trim(shapexValue);

			Shapex[strtol(unicode.data(),&endptr,16) - 0x600] = atoi(shapexValue.data());

		}
	}
	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}



	/*********************************/
	return true;
}


SpellChecker::~SpellChecker()
{

	unigramPLReader.Close();
	bigramPLReader.Close();

};

